/******************************************************************************
*
*  This file contains an abstraction of the bitstream I/O used
*  by the MPEG decoder. It will map high-level calls to the
*  appropiate file I/O provided by the underlying OS.
*
******************************************************************************/

/* Include files */
#include "mp3dec.h"

/* Local functions and variables */
static Biobuf *bio;
static int	eof;

/******************************************************************************
*
* Description: Opens a mp3 file with the given name.
* Parameters: Name of the file.
* Return value: OK if file open succeeded, ERROR otherwise.
*
******************************************************************************/
STATUS MPG_Stream_Open( const char *filename )
{
	bio = Bopen( filename, OREAD );
	if ( bio == nil ) {
		return( ERROR );
	}
	return( OK );
}


/******************************************************************************
*
* Description: Closes the file.
* Parameters: None.
* Return value: None.
*
******************************************************************************/
void MPG_Stream_Close( void )
{
	if ( bio == nil )
		return;

	Bterm( bio );
}


/******************************************************************************
*
* Description: This function returns the next byte from the bitstream, or EOF.
*              If we're not on an byte-boundary the bits remaining until
*              the next boundary are discarded before getting that byte.
* Parameters: None
* Return value: The next byte in bitstream in the lowest 8 bits, or C_MPG_EOF.
*
******************************************************************************/
uint MPG_Get_Byte( void )
{
	uint    val;

	/* Get byte */
	val = Bgetc( bio );

	/* EOF? */
	if ( val == Beof ) {
		eof = 1;
		val = C_MPG_EOF;
	}

	/* Done */
	return( val & 0xff );
}


/******************************************************************************
*
* Description: This function reads 'no_of_bytes' bytes of data from the input
*              stream into 'data_vec[]'.
* Parameters: Number of bytes to read, vector pointer where to store them.
* Return value: OK or ERROR if the operation couldn't be performed.
*
******************************************************************************/
int	MPG_Get_Bytes( uint no_of_bytes, uint data_vec[] )
{
	int	i;
	uint    val;

	for ( i = 0; i < no_of_bytes; i++) {
		val = MPG_Get_Byte();

		if ( val == C_MPG_EOF ) {
			return( C_MPG_EOF );
		} else {
			data_vec[i] = val;
		}
	}

	return( OK );
}


/******************************************************************************
*
* Description: This function returns the current file position in bytes.
* Parameters: None
* Return value: File pos in bytes, or 0 if no file open.
*
******************************************************************************/
uint MPG_Get_Filepos( void )
{
	/* File open? */
	if ( bio == nil ) {
		return( 0 );
	}
	if ( eof ) {
		return( C_MPG_EOF );
	}
	return( (uint)Boffset( bio ) );
}


